#!/usr/local/bin/perl

# Copyright (c) 2002                      RIPE NCC
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted, provided
# that the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the author not be used in advertising or
# publicity pertaining to distribution of the software without specific,
# written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
#
# -------------------------------------------------------------------------------
# Filename          : RIB-stat.pl
# Author            : Olaf Maennel <olaf@maennel.net>
#		      Vinay Aggarwal <vinay@net.in.tum.de>
# 		      Arife Vural <arife@ripe.net>
# Purpose           : RIB-stat.pl is written to build up RIB status in given time by 
#		      using the RIS rawdata. To use this tool you need to have 
#		      route_btoa (MRT) or bgpdump from http://www.ris.ripe.net/source/.
#		      Let us know if you have any comments about it. E-mail address 
#		      you can contact ris\@ripe.net.
# Date              : 2004-03-08 
# Revision          : 1.0
# Language 	    : Perl v5.8.0
# OSs Tested        : Linux 2.4.23
# -------------------------------------------------------------------------------

use strict;
use IO::Handle;
use FileHandle;
use Getopt::Std;
use Time::Local;

$ENV{PATH}.="/usr/bin:/bin:sbin";

# route_btoa or bgpdump is rquired to read rawdata.
# bgpdump can be downloaded from http://www.ris.ripe.net/source/

my $BGPDUMP="/usr/local/bin/bgpdump -m";
my $WGET="wget -T 60";
my $TMPDIR="/tmp";
my $SERVER="http://data.ris.ripe.net";

# main 
{
	my (%opt, $line, $update);
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday);
	my ($eyear, $emonth, $eday, $ehour, $emin, $esec);
	my ($i, $j, $rib_key, $dirname, $pid, $result, $bhour, $bfile);
	my ($ufile, $file, @file_arry, $rrcname);
	my %vrt = ();  #my Virtual Routing Table (VRT)
	my (@ribline, $update_info, @updline, $end_time, $enday);

	getopts('t:r:mhHp', \%opt);

	if ($opt{p}) 
	{
		print "
List of remote route collectors:

	RRC00 -- RIPE NCC, Amsterdam
	RRC01 -- LINX, London
	RRC02 -- SFINX, Paris
	RRC03 -- AMS-IX, Amsterdam
	RRC04 -- CIXP, Geneva
	RRC05 -- VIX, Vienna
	RRC06 -- NSPIXP2, Otemachi
	RRC07 -- Netnod, Stockholm
	RRC08 -- MAE-West, San Jose
	RRC09 -- TIX, Zurich
	RRC10 -- MIX, Milan
	RRC11 -- NYIIX, New York

";
	exit;
	}


	if ($opt{h} || $opt{H}) {
			usage();}
	
	if ($opt{t} =~ m/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/) {
		$eyear = $1; $emonth = $2; $eday = $3;
		$ehour = $4; $emin = $5; $esec = $6; }
	else {
		print "\nPlease specify the time as YYYYmmDDHHMMss.\n"; usage(); }

	$end_time = timegm($esec, $emin, $ehour, $eday, ($emonth - 1), ($eyear - 1900));

	if ($opt{r}) {
		$rrcname = $opt{r};
		$rrcname =~ tr/[A-Z]/[a-z]/; }
	else {
		$rrcname = "rrc00"; }

	$dirname = sprintf("%s.%s", $eyear, $emonth);
	$enday = sprintf("%s%s%s", $eyear, $emonth, $eday);

	# create tmp dir in /tmp for rawdata files
	$pid = getppid;

	$result = system("rm -rf $TMPDIR/$pid; mkdir $TMPDIR/$pid");

	if ($result) {
		print "ERROR: There is a problem to use resources on local server.\n";
		exit; }

	if ($ehour < 8) {
		$bhour = 0;
		$bfile = sprintf("bview.%s.0000", $enday); }
	elsif ($ehour < 16) {
		$bhour = 8;
		$bfile = sprintf("bview.%s.0800", $enday); }
	else {
		$bhour = 16;
		$bfile = sprintf("bview.%s.1600", $enday); }

	# open log file
	open(LOG, "> $TMPDIR/rib.log");

	$result=system("$WGET -P $TMPDIR/$pid $SERVER/$rrcname/$dirname/${bfile}.gz");

	if ($result) {
		$result=system("$WGET -P $TMPDIR/$pid $SERVER/$rrcname/$dirname/${bfile}");
		if ($result) {
			print "ERROR: $bfile.gz does not exist on $SERVER or not accessible by now.\n"; 
			exit; }
	}

	# get update files

	my $MAX;

	$bhour = $bhour + 0; $ehour = $ehour + 0;
	while ($bhour <= $ehour) 
	{
		if ($bhour == $ehour) {
			$MAX=$emin; }
		else {
			$MAX=59;}
		for ($i=0; $i <= $MAX ; $i=$i+5) 
		{
			$ufile = sprintf("updates.%06d.%02d%02d.gz", $enday, $bhour, $i);
			$result=system("$WGET -P $TMPDIR/$pid $SERVER/$rrcname/$dirname/${ufile}"); 
			if ($result) {
				$ufile = sprintf("updates.%06d.%02d%02d", $enday, $bhour, $i);
				$result=system("$WGET -P $TMPDIR/$pid $SERVER/$rrcname/$dirname/${ufile}"); 
				if ($result) {
					print LOG "$ufile is not available.\n";}
			}
		}
		$bhour++;
	}

	# uncompress the files
	system("gunzip $TMPDIR/$pid/*.gz");
	system("ls $TMPDIR/$pid/*[0-9]* > $TMPDIR/$pid/filelist");

	open (FILELIST, "$TMPDIR/$pid/filelist");

	while ($file = <FILELIST>) {
		chomp ($file);
		push (@file_arry, $file); }

	close (FILELIST);
	system("rm $TMPDIR/$pid/filelist");

	foreach $i (@file_arry) 
	{
		$result = system ("$BGPDUMP $i > $i.txt");
		if ($result == 0) {
			system ("rm $i"); }
	}


	# read into Virtual Routing Table (VRT)
	# 2-D hash, peer = 1st key, prefix = 2nd key, Announcement info = value!

	# this subroutine reads the entire table dump into %vrt hash -- bview
	# assumed first entry of array is bview file

	open (TABLE, "< $file_arry[0].txt") || print "ERROR: There is a problem reading RIB file.\n";
    
    	while (<TABLE>) 
	{
		chomp;
		(@ribline) = split /\|/;
		# 3-peer, 5-prefix, 6..- update dependant info

		$rib_key = "$ribline[3]-$ribline[5]";

		$vrt{$rib_key}{'inetype'} = $ribline[0];
		$vrt{$rib_key}{'updtime'} = $ribline[1];
		$vrt{$rib_key}{'updtype'} = $ribline[2];
		$vrt{$rib_key}{'peer'} = $ribline[3];
		$vrt{$rib_key}{'as'} = $ribline[4];
		$vrt{$rib_key}{'prefix'} = $ribline[5];
		$vrt{$rib_key}{'aspath'} = $ribline[6];
		$vrt{$rib_key}{'origin'} = $ribline[7];
		$vrt{$rib_key}{'nexthop'} = $ribline[8];
		$vrt{$rib_key}{'community'} = $ribline[11];
    	}  
    
    	# VRT has been formed completely.
    
    	close (TABLE);
    
	# process updates over RIB

	for ($i=1; $i<$#file_arry; $i++) 
	{
		open (UPDATE, "< $file_arry[$i].txt") || print "ERROR: There is a problem to open update file.\n";  
		@updline = ();
		$update = <UPDATE>;
		chomp ($update);
		@updline = split /\|/, $update;

		while (($updline[1] <= $end_time) && ($update))  
		{
    			# 1st condition ensures that only the required updates until the $end_time are read
			# 2nd condition ensures end of loop if $end_time exceeds EOF

			# modify the VRT entry

			$rib_key = "$updline[3]-$updline[5]";
    
    			if ($updline[2] eq "A")  {
				# take the 'Announcement' information from the update, and store in VRT

				# print LOG "A: $vrt{$rib_key}{'prefix'}, $vrt{$rib_key}{'peer'}, $vrt{$rib_key}{'updtime'} is replaced by ";
				# print LOG "$updline[5], $updline[3], $updline[1].\n";

				$vrt{$rib_key}{'inetype'} = $updline[0];
				$vrt{$rib_key}{'updtime'} = $updline[1];
				$vrt{$rib_key}{'updtype'} = $updline[2];
				$vrt{$rib_key}{'peer'} = $updline[3];
				$vrt{$rib_key}{'as'} = $updline[4];
				$vrt{$rib_key}{'prefix'} = $updline[5];
				$vrt{$rib_key}{'aspath'} = $updline[6];
				$vrt{$rib_key}{'origin'} = $updline[7];
				$vrt{$rib_key}{'nexthop'} = $updline[8];


    			}  # end if (A)
    
    			if ($updline[2] eq "W") {
				if ( exists $vrt{$rib_key} ) {
					# print LOG "W: $vrt{$rib_key}{'prefix'}, $vrt{$rib_key}{'peer'}, $vrt{$rib_key}{'updtime'} is deleted.\n";
    					delete $vrt{$rib_key}; }
			}  # end if (W)
   
   			# Look at the state changes
			# if state changes from 6 to 1, delete all the
			# entries for the peer

    			if ($updline[2] eq "STATE") {
				if (($updline[5] == 6) && ($updline[6] == 1)) {
					foreach $j (keys %vrt) {
						if ($vrt{$j}{'peer'} eq $updline[3]) {
							print # LOG "STATE: $j is deleted.\n";
			    				delete $vrt{$j}; }
					}
				}
    			}  # end if STATE changes
    
    			$update = <UPDATE>;
    			chomp ($update);
    			@updline = split /\|/, $update;
    
		}  #end while (($updline[1] <= $end_time) && ($update))
		close (UPDATE);
	}

	my ($outsday, $outeday);

	$opt{t} =~ m/(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/;
	$outeday=sprintf("%02d/%02d/%02d %02d:%02d:%02d", $3, $4, $2, $5, $6, $7);

	print "\nRIB status is seen at $outeday on $rrcname.\n\n";

	if ($opt{m})
	{
		print "TIME|TYPE|PREFIX|PEER|AS|ORIGIN|ASPATH|NEXTHOP|COMMUNITY\n";
		foreach $i (keys %vrt) {
			print "$vrt{$i}{'updtime'}|$vrt{$i}{'updtype'}|$vrt{$i}{'prefix'}|$vrt{$i}{'peer'}|$vrt{$i}{'as'}|$vrt{$i}{'origin'}|";
			print "$vrt{$i}{'aspath'}|$vrt{$i}{'nexthop'}|$vrt{$i}{'community'}\n";
		}
	}
	else
	{
		foreach $i (sort {$vrt{$a}{'updtime'} <=> $vrt{$b}{'updtime'}} keys %vrt) 
		{
			($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = gmtime($vrt{$i}{'updtime'});
			$mon = ($mon + 1);
			$year = ($year + 1900); $year =~ m/(\d{2})(\d{2})/; $year = $2;

			printf("TIME: %02d/%02d/%02d %02d:%02d:%02d\n", $mon, $mday, $year, $hour, $min, $sec);
			if ($vrt{$i}{'prefix'} =~ m/:/) {
				printf("TYPE: TABLE_DUMP/INET6\n"); }
			else {
				printf("TYPE: TABLE_DUMP/INET\n"); }
			printf("PREFIX: %s\n", $vrt{$i}{'prefix'});
			printf("FROM: %s AS%s\n", $vrt{$i}{'peer'}, $vrt{$i}{'as'});
			printf("ORIGIN: %s\n", $vrt{$i}{'origin'});
			printf("ASPATH: %s\n", $vrt{$i}{'aspath'});
			printf("NEXTHOP: %s\n", $vrt{$i}{'nexthop'});
			if ($vrt{$i}{'community'}) {
				printf("COMMUNITY: %s\n\n", $vrt{$i}{'community'}); }
			else { 
				print "COMMUNITY: \n\n"; }
		}
	}

	# clean up the rawdata files
	system ("rm -rf $TMPDIR/$pid");

	close (LOG);
}

sub usage()
{
	print "
Usage: RIB-stat

	-t:  time (YYYYmmDDHHMMss)
	-r:  name of rrc box like rrc00, rrc01 or so.
	     rrc00 by default.
	-m:  produce output in machine readable format.
	-h:  get help
	-p:  print list of RRCs

";
	exit(0);

}
